@@ -20,6 +20,7 @@ module XMonad.Actions.SpawnOn (
     -- $usage
     Spawner,
     manageSpawn,
+    manageSpawnWithGC,
     spawnHere,
     spawnOn,
     spawnAndDo,
@@ -44,16 +45,16 @@ import qualified XMonad.Util.ExtensibleState as XS
 -- >    import XMonad.Actions.SpawnOn
 --
 -- >    main = do
--- >      xmonad defaultConfig {
+-- >      xmonad def {
 -- >         ...
--- >         manageHook = manageSpawn <+> manageHook defaultConfig
+-- >         manageHook = manageSpawn <+> manageHook def
 -- >         ...
 -- >      }
 --
 -- To ensure that application appears on a workspace it was launched at, add keybindings like:
 --
 -- >  , ((mod1Mask,xK_o), spawnHere "urxvt")
--- >  , ((mod1Mask,xK_s), shellPromptHere defaultXPConfig)
+-- >  , ((mod1Mask,xK_s), shellPromptHere def)
 --
 -- The module can also be used to apply other manage hooks to the window of
 -- the spawned application(e.g. float or resize it).
@@ -66,8 +67,6 @@ newtype Spawner = Spawner {pidsRef :: [(ProcessID, ManageHook)]} deriving Typeab
 instance ExtensionClass Spawner where
     initialValue = Spawner []
 
-maxPids :: Int
-maxPids = 5
 
 -- | Get the current Spawner or create one if it doesn't exist.
 modifySpawner :: ([(ProcessID, ManageHook)] -> [(ProcessID, ManageHook)]) -> X ()
@@ -76,20 +75,26 @@ modifySpawner f = XS.modify (Spawner . f . pidsRef)
 -- | Provides a manage hook to react on process spawned with
 -- 'spawnOn', 'spawnHere' etc.
 manageSpawn :: ManageHook
-manageSpawn = do
+manageSpawn = manageSpawnWithGC (return . take 20)
+
+manageSpawnWithGC :: ([(ProcessID, ManageHook)] -> X [(ProcessID, ManageHook)])
+        -- ^ function to stop accumulation of entries for windows that never set @_NET_WM_PID@
+       -> ManageHook
+manageSpawnWithGC garbageCollect = do
     Spawner pids <- liftX XS.get
     mp <- pid
     case flip lookup pids =<< mp of
         Nothing -> idHook
         Just mh  -> do
-            whenJust mp $ \p ->
-                liftX . modifySpawner $ filter ((/= p) . fst)
+            whenJust mp $ \p -> liftX $ do
+                ps <- XS.gets pidsRef
+                XS.put . Spawner =<< garbageCollect (filter ((/= p) . fst) ps)
             mh
 
 mkPrompt :: (String -> X ()) -> XPConfig -> X ()
 mkPrompt cb c = do
     cmds <- io $ getCommands
-    mkXPrompt Shell c (getShellCompl cmds) cb
+    mkXPrompt Shell c (getShellCompl cmds $ searchPredicate c) cb
 
 -- | Replacement for Shell prompt ("XMonad.Prompt.Shell") which launches
 -- application on current workspace.
@@ -115,7 +120,7 @@ spawnOn ws cmd = spawnAndDo (doShift ws) cmd
 spawnAndDo :: ManageHook -> String -> X ()
 spawnAndDo mh cmd = do
     p <- spawnPID $ mangle cmd
-    modifySpawner $ (take maxPids . ((p,mh) :))
+    modifySpawner $ ((p,mh) :)
  where
     -- TODO this is silly, search for a better solution
     mangle xs | any (`elem` metaChars) xs || "exec" `isInfixOf` xs = xs
