@@ -4,6 +4,7 @@
 -- |
 -- Module      :  XMonad.Prompt
 -- Copyright   :  (C) 2007 Andrea Rossato, 2015 Evgeny Kurnevsky
+--                    2015 Sibi Prabakaran
 -- License     :  BSD3
 --
 -- Maintainer  :  Spencer Janssen <spencerjanssen@gmail.com>
@@ -90,7 +91,6 @@ import           Data.List
 import qualified Data.Map                     as M
 import           Data.Maybe                   (fromMaybe)
 import           Data.Set                     (fromList, toList)
-import           System.Directory             (getAppUserDataDirectory)
 import           System.IO
 import           System.Posix.Files
 
@@ -126,14 +126,14 @@ data XPState =
         }
 
 data XPConfig =
-    XPC { font              :: String     -- ^ Font
+    XPC { font              :: String     -- ^ Font; use the prefix @"xft:"@ for TrueType fonts
         , bgColor           :: String     -- ^ Background color
         , fgColor           :: String     -- ^ Font color
         , fgHLight          :: String     -- ^ Font color of a highlighted completion entry
         , bgHLight          :: String     -- ^ Background color of a highlighted completion entry
         , borderColor       :: String     -- ^ Border color
         , promptBorderWidth :: !Dimension -- ^ Border width
-        , position          :: XPPosition -- ^ Position: 'Top' or 'Bottom'
+        , position          :: XPPosition -- ^ Position: 'Top', 'Bottom', or 'CenteredAt'
         , alwaysHighlight   :: !Bool      -- ^ Always highlight an item, overriden to True with multiple modes. This implies having *one* column of autocompletions only.
         , height            :: !Dimension -- ^ Window height
         , maxComplRows      :: Maybe Dimension
@@ -144,12 +144,12 @@ data XPConfig =
                                          -- history entries to remember
         , promptKeymap      :: M.Map (KeyMask,KeySym) (XP ())
                                          -- ^ Mapping from key combinations to actions
-        , completionKey     :: KeySym     -- ^ Key that should trigger completion
+        , completionKey     :: (KeyMask, KeySym)     -- ^ Key that should trigger completion
         , changeModeKey     :: KeySym     -- ^ Key to change mode (when the prompt has multiple modes)
         , defaultText       :: String     -- ^ The text by default in the prompt line
         , autoComplete      :: Maybe Int  -- ^ Just x: if only one completion remains, auto-select it,
-        , showCompletionOnTab :: Bool     -- ^ Only show list of completions when Tab was pressed
                                           --   and delay by x microseconds
+        , showCompletionOnTab :: Bool     -- ^ Only show list of completions when Tab was pressed
         , searchPredicate   :: String -> String -> Bool
                                           -- ^ Given the typed string and a possible
                                           --   completion, is the completion valid?
@@ -228,6 +228,17 @@ class XPrompt t where
 
 data XPPosition = Top
                 | Bottom
+                -- | Prompt will be placed in the center horizontally and
+                --   in the certain place of screen vertically. If it's in the upper
+                --   part of the screen, completion window will be placed below(like
+                --   in 'Top') and otherwise above(like in 'Bottom')
+                | CenteredAt { xpCenterY :: Rational
+                             -- ^ Rational between 0 and 1, giving
+                             -- y coordinate of center of the prompt relative to the screen height.
+                             , xpWidth  :: Rational
+                             -- ^ Rational between 0 and 1, giving
+                             -- width of the prompt relatave to the screen width.
+                             }
                   deriving (Show,Read)
 
 amberXPConfig, defaultXPConfig, greenXPConfig :: XPConfig
@@ -242,7 +253,7 @@ instance Default XPConfig where
         , borderColor       = "white"
         , promptBorderWidth = 1
         , promptKeymap      = defaultXPKeymap
-        , completionKey     = xK_Tab
+        , completionKey     = (0,xK_Tab)
         , changeModeKey     = xK_grave
         , position          = Bottom
         , height            = 18
@@ -488,14 +499,15 @@ handle ks@(sym,_) e@(KeyEvent {ev_event_type = t, ev_state = m}) = do
   complKey <- gets $ completionKey . config
   chgModeKey <- gets $ changeModeKey . config
   c <- getCompletions
+  mCleaned <- cleanMask m
   when (length c > 1) $ modify (\s -> s { showComplWin = True })
-  if complKey == sym
+  if complKey == (mCleaned,sym)
      then completionHandle c ks e
      else if (sym == chgModeKey) then
            do
              modify setNextMode
              updateWindows
-          else when (t == keyPress) $ keyPressHandle m ks
+          else when (t == keyPress) $ keyPressHandle mCleaned ks
 handle _ (ExposeEvent {ev_window = w}) = do
   st <- get
   when (win st == w) updateWindows
@@ -506,25 +518,61 @@ completionHandle ::  [String] -> KeyStroke -> Event -> XP ()
 completionHandle c ks@(sym,_) (KeyEvent { ev_event_type = t, ev_state = m }) = do
   complKey <- gets $ completionKey . config
   alwaysHlight <- gets $ alwaysHighlight . config
+  mCleaned <- cleanMask m
   case () of
-    () | t == keyPress && sym == complKey ->
-          do
-            st <- get
-            let updateState l = case alwaysHlight of
-                  -- modify the buffer's value
-                  False -> let newCommand = nextCompletion (currentXPMode st) (command st) l
-                           in modify $ \s -> setCommand newCommand $ s { offset = length newCommand, highlightedCompl = Just newCommand}
-                  --TODO: Scroll or paginate results
-                  True -> let complIndex' = nextComplIndex st (length l)
-                              highlightedCompl' = highlightedItem st { complIndex = complIndex'} c
-                          in modify $ \s -> setHighlightedCompl highlightedCompl' $ s { complIndex = complIndex' }
-                updateWins l = redrawWindows l >> eventLoop (completionHandle l)
-            case c of
-              []  -> updateWindows   >> eventLoop handle
-              [x] -> updateState [x] >> getCompletions >>= updateWins
-              l   -> updateState l   >> updateWins l
-      | t == keyRelease && sym == complKey -> eventLoop (completionHandle c)
-      | otherwise -> keyPressHandle m ks -- some other key, handle it normally
+    () | t == keyPress && (mCleaned,sym) == complKey -> do
+           st <- get
+
+           let updateWins  l = redrawWindows l >> eventLoop (completionHandle l)
+               updateState l = case alwaysHlight of
+                 False                                           -> simpleComplete l st
+                 True | Just (command st) /= highlightedCompl st -> alwaysHighlightCurrent st
+                      | otherwise                                -> alwaysHighlightNext l st
+
+           case c of
+             []  -> updateWindows   >> eventLoop handle
+             [x] -> updateState [x] >> getCompletions >>= updateWins
+             l   -> updateState l   >> updateWins l
+      | t == keyRelease && (mCleaned,sym) == complKey -> eventLoop (completionHandle c)
+      | otherwise -> keyPressHandle mCleaned ks -- some other key, handle it normally
+  where
+    -- When alwaysHighlight is off, just complete based on what the
+    -- user has typed so far.
+    simpleComplete :: [String] -> XPState -> XP ()
+    simpleComplete l st = do
+      let newCommand = nextCompletion (currentXPMode st) (command st) l
+      modify $ \s -> setCommand newCommand $
+                     s { offset = length newCommand
+                       , highlightedCompl = Just newCommand
+                       }
+
+    -- If alwaysHighlight is on, and this is the first use of the
+    -- completion key, update the buffer so that it contains the
+    -- current completion item.
+    alwaysHighlightCurrent :: XPState -> XP ()
+    alwaysHighlightCurrent st = do
+      let newCommand = fromMaybe (command st) $ highlightedItem st c
+      modify $ \s -> setCommand newCommand $
+                     setHighlightedCompl (Just newCommand) $
+                     s { offset = length newCommand
+                       }
+
+    -- If alwaysHighlight is on, and the user wants the next
+    -- completion, move to the next completion item and update the
+    -- buffer to reflect that.
+    --
+    --TODO: Scroll or paginate results
+    alwaysHighlightNext :: [String] -> XPState -> XP ()
+    alwaysHighlightNext l st = do
+      let complIndex' = nextComplIndex st (length l)
+          highlightedCompl' = highlightedItem st { complIndex = complIndex'} c
+          newCommand = fromMaybe (command st) $ highlightedCompl'
+      modify $ \s -> setHighlightedCompl highlightedCompl' $
+                     setCommand newCommand $
+                     s { complIndex = complIndex'
+                       , offset = length newCommand
+                       }
+
 -- some other event: go back to main loop
 completionHandle _ k e = handle k e
 
@@ -662,12 +710,11 @@ emacsLikeXPKeymap' p = M.fromList $
 keyPressHandle :: KeyMask -> KeyStroke -> XP ()
 keyPressHandle m (ks,str) = do
   km <- gets (promptKeymap . config)
-  kmask <- cleanMask m -- mask is defined in ghc7
-  case M.lookup (kmask,ks) km of
+  case M.lookup (m,ks) km of
     Just action -> action >> updateWindows
     Nothing -> case str of
                  "" -> eventLoop handle
-                 _ -> when (kmask .&. controlMask == 0) $ do
+                 _ -> when (m .&. controlMask == 0) $ do
                                  let str' = if isUTF8Encoded str
                                                then decodeString str
                                                else str
@@ -842,8 +889,12 @@ createWin d rw c s = do
   let (x,y) = case position c of
                 Top -> (0,0)
                 Bottom -> (0, rect_height s - height c)
+                CenteredAt py w -> (floor $ (fi $ rect_width s) * ((1 - w) / 2), floor $ py * fi (rect_height s) - (fi (height c) / 2))
+      width = case position c of
+                CenteredAt _ w -> floor $ fi (rect_width s) * w
+                _              -> rect_width s
   w <- mkUnmanagedWindow d (defaultScreenOfDisplay d) rw
-                      (rect_x s + x) (rect_y s + fi y) (rect_width s) (height c)
+                      (rect_x s + x) (rect_y s + fi y) width (height c)
   mapWindow d w
   return w
 
@@ -852,7 +903,9 @@ drawWin = do
   st <- get
   let (c,(d,(w,gc))) = (config &&& dpy &&& win &&& gcon) st
       scr = defaultScreenOfDisplay d
-      wh = widthOfScreen scr
+      wh = case position c of
+             CenteredAt _ wd -> floor $ wd * fi (widthOfScreen scr)
+             _               -> widthOfScreen scr
       ht = height c
       bw = promptBorderWidth c
   Just bgcolor <- io $ initColor d (bgColor c)
@@ -935,8 +988,11 @@ getComplWinDim :: [String] -> XP ComplWindowDim
 getComplWinDim compl = do
   st <- get
   let (c,(scr,fs)) = (config &&& screen &&& fontS) st
-      wh = rect_width scr
+      wh = case position c of
+             CenteredAt _ w -> floor $ fi (rect_width scr) * w
+             _ -> rect_width scr
       ht = height c
+      bw = promptBorderWidth c
 
   tws <- mapM (textWidthXMF (dpy st) fs) compl
   let max_compl_len =  fromIntegral ((fi ht `div` 2) + maximum tws)
@@ -951,8 +1007,11 @@ getComplWinDim compl = do
       actual_rows = min actual_max_number_of_rows (fi needed_rows)
       actual_height = actual_rows * ht
       (x,y) = case position c of
-                Top -> (0,ht)
-                Bottom -> (0, (0 + rem_height - actual_height))
+                Top -> (0,ht - bw)
+                Bottom -> (0, (0 + rem_height - actual_height + bw))
+                CenteredAt py w
+                  | py <= 1/2 -> (floor $ fi (rect_width scr) * ((1 - w) / 2), floor (py * fi (rect_height scr) + (fi ht)/2) - bw)
+                  | otherwise -> (floor $ fi (rect_width scr) * ((1 - w) / 2), floor (py * fi (rect_height scr) - (fi ht)/2) - actual_height + bw)
   (asc,desc) <- io $ textExtentsXMF fs $ head compl
   let yp = fi $ (ht + fi (asc - desc)) `div` 2
       xp = (asc + desc) `div` 2
@@ -1039,7 +1098,7 @@ emptyHistory :: History
 emptyHistory = M.empty
 
 getHistoryFile :: IO FilePath
-getHistoryFile = fmap (++ "/history") $ getAppUserDataDirectory "xmonad"
+getHistoryFile = fmap (++ "/prompt-history") getXMonadCacheDir
 
 readHistory :: IO History
 readHistory = readHist `E.catch` \(SomeException _) -> return emptyHistory
@@ -1145,7 +1204,7 @@ breakAtSpace s
 
 -- | 'historyCompletion' provides a canned completion function much like
 --   'getShellCompl'; you pass it to mkXPrompt, and it will make completions work
---   from the query history stored in ~\/.xmonad\/history.
+--   from the query history stored in the XMonad cache directory.
 historyCompletion :: ComplFunction
 historyCompletion = historyCompletionP (const True)
 
